package repo

import (
	"database/sql"
	"encoding/json"
	"testing"
	"time"

	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/adapter/repo/sqlc"
	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/model"
	"github.com/stretchr/testify/assert"
)

func TestConvToExpenseRetrieved(t *testing.T) {
	input := &sqlc.GetExpenseByIDRow{
		ID:        123,
		CreatedAt: time.Date(2000, time.April, 11, 0, 0, 0, 0, time.UTC),
		UpdatedAt: time.Date(2000, time.April, 11, 0, 0, 0, 0, time.UTC),
		Amount:    123,
		Currency:  "EUR",
		EventID:   123,
		Name:      sql.NullString{Valid: false},
		Place:     sql.NullString{Valid: false},
		Payments: json.RawMessage(
			`[{"payer_id": 1, "payer_first_name": "toto", "payer_last_name": "titi", "amount": 10, "currency": "EUR"},
            {"payer_id": 2, "payer_first_name": "tata", "payer_last_name": "titi", "amount": 10, "currency": "EUR"}]`,
		),
		Benefits: json.RawMessage(
			`[{"recipient_id": 1, "recipient_first_name": "toto", "recipient_last_name": "titi", "amount": 10, "currency": "EUR"},
            {"recipient_id": 2, "recipient_first_name": "tata", "recipient_last_name": "titi", "amount": 10, "currency": "EUR"}]`,
		),
	}

	want := &model.ExpenseRetrieved{
		ID:        123,
		CreatedAt: time.Date(2000, time.April, 11, 0, 0, 0, 0, time.UTC),
		UpdatedAt: time.Date(2000, time.April, 11, 0, 0, 0, 0, time.UTC),
		Amount:    model.Money{Amount: 123, Currency: model.Currency("EUR")},
		EventID:   123,
		Detail:    model.ExpenseDetail{},
		Payments: []model.Payment{
			{
				PayerID:        1,
				PayerFirstName: "toto",
				PayerLastName:  "titi",
				Amount:         model.Money{Amount: 10, Currency: model.Currency("EUR")},
			},
			{
				PayerID:        2,
				PayerFirstName: "tata",
				PayerLastName:  "titi",
				Amount:         model.Money{Amount: 10, Currency: model.Currency("EUR")},
			},
		},
		Benefits: []model.Benefit{
			{
				RecipientID:        1,
				RecipientFirstName: "toto",
				RecipientLastName:  "titi",
				Amount:             model.Money{Amount: 10, Currency: model.Currency("EUR")},
			},
			{
				RecipientID:        2,
				RecipientFirstName: "tata",
				RecipientLastName:  "titi",
				Amount:             model.Money{Amount: 10, Currency: model.Currency("EUR")},
			},
		},
	}

	got, err := convToExpenseRetrieved(input)
	assert.NoError(t, err)
	assert.Equal(t, want, got)
}
